from .base import Widget


class Button(Widget):
    def create(self):
        self.native = self._create_native_widget(
            "button",
            classes=["btn-block"],
        )
        self.native.onclick = self.dom_onclick

    def dom_onclick(self, event):
        if self.interface.on_press:
            self.interface.on_press(self.interface)

    def set_text(self, text):
        self.native.innerHTML = text

    def set_enabled(self, value):
        self.native.disabled = not value

    def set_background_color(self, value):
        pass

    def set_on_press(self, handler):
        pass

    def rehint(self):
        pass



import asyncio

from toga_web.libs import js


class BaseDialog:
    def __init__(self):
        loop = asyncio.get_event_loop()
        self.future = loop.create_future()

    def __eq__(self, other):
        raise RuntimeError(
            "Can't check dialog result directly; use await or an on_result handler"
        )

    def __bool__(self):
        raise RuntimeError(
            "Can't check dialog result directly; use await or an on_result handler"
        )

    def __await__(self):
        return self.future.__await__()


class InfoDialog(BaseDialog):
    def __init__(self, window, title, message, on_result=None):
        super().__init__()
        # TODO: Replace with something more customized using Bootstrap modals.
        js.alert(message)

        self.future.set_result(None)


class QuestionDialog(BaseDialog):
    def __init__(self, window, title, message, on_result=None):
        super().__init__()

        # TODO: Replace with something more customized using Bootstrap modals.
        self.future.set_result(js.confirm(message))


class ConfirmDialog(BaseDialog):
    def __init__(self, window, title, message, on_result=None):
        super().__init__()

        # TODO: Replace with something more customized using Bootstrap modals.
        self.future.set_result(js.confirm(message))


class ErrorDialog(BaseDialog):
    def __init__(self, window, title, message, on_result=None):
        super().__init__()

        # TODO: Replace with something more customized using Bootstrap modals.
        self.future.set_result(js.alert(message))


class StackTraceDialog:
    def __init__(self, window, title, message, on_result=None, **kwargs):
        super().__init__()
        # TODO: Replace with something more customized using Bootstrap modals.
        if kwargs.get("retry"):
            self.future.set_result(
                js.confirm("Stack trace: \n\n:" + message + "\n\nRetry?")
            )
        else:
            self.future.set_result(js.alert("Stack trace: \n\n:" + message))


class SaveFileDialog:
    def __init__(
        self,
        window,
        title,
        filename,
        initial_directory,
        file_types=None,
        on_result=None,
    ):
        window.factory.not_implemented("Window.save_file_dialog()")


class OpenFileDialog:
    def __init__(
        self, window, title, initial_directory, file_types, multiselect, on_result=None
    ):
        window.factory.not_implemented("Window.open_file_dialog()")


class SelectFolderDialog:
    def __init__(self, window, title, initial_directory, multiselect, on_result=None):
        window.factory.not_implemented("Window.select_folder_dialog()")



from pathlib import Path

import toga
from toga import App


class Paths:
    # Allow instantiating Path object via the factory
    Path = Path

    @property
    def app(self):
        # FIXME: None of the paths are right in a web context.
        # return Path(__main__.__file__).parent
        return Path("/")

    @property
    def data(self):
        return Path.home() / ".local" / "share" / App.app.name

    @property
    def cache(self):
        return Path.home() / ".cache" / App.app.name

    @property
    def logs(self):
        return Path.home() / ".cache" / App.app.name / "log"

    @property
    def toga(self):
        """Return a path to a Toga resources."""
        return Path(toga.__file__).parent


paths = Paths()



import toga
from toga_web.libs import create_element, js
from toga_web.window import Window


class MainWindow(Window):
    def on_close(self, *args):
        pass


class App:
    def __init__(self, interface):
        self.interface = interface
        self.interface._impl = self

    def create(self):

        # self.resource_path = os.path.dirname(os.path.dirname(NSBundle.mainBundle.bundlePath))

        formal_name = self.interface.formal_name

        self.interface.commands.add(
            # ---- Help menu ----------------------------------
            toga.Command(
                lambda _: self.interface.about(),
                "About " + formal_name,
                group=toga.Group.HELP,
            ),
            toga.Command(
                None,
                "Preferences",
                group=toga.Group.HELP,
            ),
        )

        # Create the menus.
        self.create_menus()

        # Call user code to populate the main window
        self.interface.startup()

    def _create_submenu(self, group, items):
        submenu = create_element(
            "li",
            classes=["nav-item", "dropdown"],
            children=[
                create_element(
                    "a",
                    id=f"menu-{id(group)}",
                    classes=[
                        "nav-link",
                        "dropdown-toggle",
                    ],
                    data_toggle="dropdown",
                    aria_haspopup="true",
                    aria_expanded="false",
                    content=group.text,
                ),
                create_element(
                    "div",
                    classes=["dropdown-menu"],
                    aria_labelledby=f"menu-{id(group)}",
                    children=items,
                ),
            ],
        )
        return submenu

    def create_menus(self):
        self._menu_groups = {}
        submenu = None

        for cmd in self.interface.commands:
            if cmd == toga.GROUP_BREAK:
                submenu = None
            elif cmd == toga.SECTION_BREAK:
                # TODO - add a section break
                pass
            else:
                # TODO - this doesn't handle submenus properly;
                # all menu items will appear in their root group.
                submenu = self._menu_groups.setdefault(cmd.group, [])

                menu_item = create_element(
                    "a",
                    classes=["dropdown-item"] + ([] if cmd.enabled else ["disabled"]),
                    content=cmd.text,
                )
                menu_item.onclick = cmd.action

                submenu.append(menu_item)

        menu_container = create_element(
            "ul",
            classes=["navbar-nav", "mr-auto"],
        )
        help_menu_container = create_element(
            "ul",
            classes=["navbar-nav", "ml-auto"],
        )

        for group, items in self._menu_groups.items():
            submenu = self._create_submenu(group, items)
            if group != toga.Group.HELP:
                menu_container.appendChild(submenu)
            else:
                help_menu_container.appendChild(submenu)

        self.menubar = create_element(
            "header",
            children=[
                create_element(
                    "nav",
                    classes=[
                        "navbar",
                        "fixed-top",
                        "navbar-expand-lg",
                        "navbar-dark",
                        "bg-dark",
                    ],
                    children=[
                        create_element(
                            "div",
                            classes=["container"],
                            children=[
                                create_element(
                                    "a",
                                    classes=["navbar-brand"],
                                    content=f"""
                                        <img src="static/logo-32.png"
                                             class="d-inline-block align-top"
                                             alt="{self.interface.formal_name} logo"
                                             loading="lazy">
                                        {self.interface.formal_name}
                                    """,
                                ),
                                create_element(
                                    "button",
                                    classes=["navbar-toggler"],
                                    type="button",
                                    data_toggle="collapse",
                                    data_target="#navbarsExample07",
                                    aria_controls="navbarsExample07",
                                    aria_expanded="false",
                                    aria_label="Toggle navigation",
                                    children=[
                                        create_element(
                                            "span", classes=["navbar-toggler-icon"]
                                        )
                                    ],
                                ),
                                create_element(
                                    "div",
                                    id="navbarsExample07",
                                    classes=["collapse", "navbar-collapse"],
                                    children=[
                                        menu_container,
                                        help_menu_container,
                                    ],
                                ),
                            ],
                        )
                    ],
                )
            ],
        )

        # Menubar exists at the app level.
        app_placeholder = js.document.getElementById("app-placeholder")
        app_placeholder.appendChild(self.menubar)

    def main_loop(self):
        self.create()

    def set_main_window(self, window):
        pass

    def show_about_dialog(self):
        about_text = f"{self.interface.formal_name}"

        if self.interface.version is not None:
            about_text += f" v{self.interface.version}"

        if self.interface.author is not None:
            about_text += "\n\nCopyright © {author}".format(
                author=self.interface.author
            )

        js.alert(about_text)

    def exit(self):
        pass

    def set_on_exit(self, value):
        pass

    def current_window(self):
        self.interface.factory.not_implemented("App.current_window()")

    def enter_full_screen(self, windows):
        self.interface.factory.not_implemented("App.enter_full_screen()")

    def exit_full_screen(self, windows):
        self.interface.factory.not_implemented("App.exit_full_screen()")

    def show_cursor(self):
        self.interface.factory.not_implemented("App.show_cursor()")

    def hide_cursor(self):
        self.interface.factory.not_implemented("App.hide_cursor()")



from toga_web.libs import create_element, js


class WebViewport:
    def __init__(self):
        self.dpi = 96
        self.baseline_dpi = 96

    @property
    def width(self):
        return 1024

    @property
    def height(self):
        return 768


class Window:
    def __init__(self, interface, title, position, size):
        self.interface = interface
        self.interface._impl = self

        self.native = create_element(
            "main",
            id=f"toga_{self.interface.id}",
            classes=["toga", "window", "container"],
            role="main",
        )

        app_placeholder = js.document.getElementById("app-placeholder")
        app_placeholder.appendChild(self.native)

        self.set_title(title)

    def get_title(self):
        return js.document.title

    def set_title(self, title):
        js.document.title = title

    def set_app(self, app):
        pass

    def create_toolbar(self):
        self.interface.factory.not_implemented("Window.create_toolbar()")

    def clear_content(self):
        if self.interface.content:
            for child in self.interface.content.children:
                child._impl.container = None

    def set_content(self, widget):
        widget.viewport = WebViewport()

        # Remove existing content of the window.
        for child in self.native.childNodes:
            self.native.removeChild(child)

        # Add all children to the content widget.
        self.native.appendChild(widget.native)

    def show(self):
        self.native.style = "visibility: visible;"

    def hide(self):
        self.native.style = "visibility: hidden;"

    def get_visible(self):
        self.interface.not_implemented("Window.get_visible()")

    def on_close(self, *args):
        pass

    def on_size_allocate(self, widget, allocation):
        pass

    def close(self):
        self.interface.factory.not_implemented("Window.close()")

    def get_position(self):
        return (0, 0)

    def set_position(self, position):
        # Does nothing on web
        pass

    def get_size(self):
        return (self.content.viewport.width, self.content.viewport.height)

    def set_size(self, size):
        # Does nothing on web
        pass

    def set_full_screen(self, is_full_screen):
        self.interface.factory.not_implemented("Window.set_full_screen()")

    def set_on_close(self, handler):
        pass



try:
    # Try to import js from the PyScript namespace.
    import js
except ModuleNotFoundError:
    # To ensure the code can be imported, provide a js symbol
    # as a fallback
    js = None


def create_element(
    tag, id=None, classes=None, style=None, content=None, children=None, **properties
):
    """Utility method for creating DOM elements.

    :param tag: The HTML tag of the element to create
    :param id: (Optional) The ID of the new element
    :param classes: (Optional) A list of classes to attach to the
        new element.
    :param style: (Optional) The CSS style for the element.
    :param content: (Optional) The innerHTML content of the element.
    :param children: (Optional) A list of direct descendents to add to
        the element.
    :param properties: Any additional properties that should be set.
        These *must* be HTML DOM properties (e.g., ``readOnly``);
        they cannot be events or methods.
    :returns: A newly created DOM element.
    """
    element = js.document.createElement(tag)

    if id:
        element.id = id

    if classes:
        for klass in classes:
            element.classList.add(klass)

    if style:
        element.style = style

    for attr, value in properties.items():
        element.setAttribute(attr.replace("_", "-"), value)

    if content:
        element.innerHTML = content

    if children:
        for child in children:
            element.appendChild(child)

    return element
